function [U] = FindCompatPerms(pval,base,T)
%FINDCOMPATPERMS enumerates all permutations in a group G that are 
%compatible with a place valuation.
%
% DESCRIPTION:
% A permutation g1*g2*...*gn (* is the composition operation) in G, where
% gj is a coset representative in Tj for each 1<=j<=n is compatible with a
% valuation pval:X->N if pval(g1*...*gi)(xi) belongs to the image of a
% multiset selector select(xi,Ti) for each 1<=u<=n.
%
% This function is an implementation of Algorithm 4.1 in ref 1. The
% multiset selector used in this implementation is the minimal element 
% version.
%
% INPUT:  pval is a numeric vector representing a valuation mapping 
%         elements of a set X to the natural numbers.
%
%         base is a numeric vector representing a base for the group G.
%
%         T is a length(base) by 1 cell array representing a strong
%         generating set for G. Each cell i contains the coset
%         representatives of the subgroup of G stabilizing base(1:i) in the
%         subgroup of G stabilizing base(1:i-1).
%
% OUTPUT: U is an m by n numeric array listing the set of m compatible
%         permutations.
%
% See also, SchreierSims, FindCompatPerms_injective
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References: 
% 1. Junttila T. On the symmetry reduction method for petri nets and 
%    similar formalisms (doctoral thesis). Helsinki, Finland: Helsinki 
%    University of Technology, 2003.

d = length(T);
n = length(pval);
U = [];
backtrack(1,1:n);
function [] = backtrack(l,g)
    if l == d+1
        U = [U;g];
        return
    end
    Tl = T{l};
    p = base(l);
    S = select(pval,p,g,Tl);
    for i = find(pval(g(Tl(:,p))) == S);
        h = Tl(i,:);
        backtrack(l+1,g(h));
    end
end
end
function [S] = select(pval,p,g,Tl)
    S = min(pval(g(Tl(:,p))));
end