function [g] = FindCompatPerms_injective(pval,base,T)
%FINDCOMPATPERMS_INJECTIVE enumerates the unique permutation that is 
%compatible with an injective function pval.
%
% DESCRIPTION:
% A permutation g1*g2*...*gn (* is the composition operation) in G, where
% gj is a coset representative in Tj for each 1<=j<=n is compatible with a
% valuation pval:X->N if pval(g1*...*gi)(xi) belongs to the image of a
% multiset selector select(xi,Ti) for each 1<=u<=n.
%
% This function is an implementation of Algorithm 4.1 in ref 1. The
% multiset selector used in this implementation is the minimal element 
% version. This is a more efficient version of the function
% FindCompatPerms, but should only be used when pval is an injective
% function.
%
% INPUT:  pval is a numeric vector representing an injective valuation 
%         mapping elements of a set X to the natural numbers.
%
%         base is a numeric vector representing a base for the group G.
%
%         T is a length(base) by 1 cell array representing a strong
%         generating set for G. Each cell i contains the coset
%         representatives of the subgroup of G stabilizing base(1:i) in the
%         subgroup of G stabilizing base(1:i-1).
%
% OUTPUT: g is a permutation of the set X representing the unique
%         permutation compatible with pval.
%
% See also, FindCompatPerms, SchreierSims
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References: 
% 1. Junttila T. On the symmetry reduction method for petri nets and 
%    similar formalisms (doctoral thesis). Helsinki, Finland: Helsinki 
%    University of Technology, 2003.

pval = pval(:);
d = length(T);
n = length(pval);
g = 1:n;
for l = 1:d
    Tl = T{l};
    p = base(l);
    [~,ix] = min(pval(g(Tl(:,p))));
    h = Tl(ix,:);
    g = g(h);
end
end