function [numdigits,startofdigit,endofdigit] = GenerateCodingScheme(f,Tsize)
%GENERATECODINGSCHEME finds the number of double-precision digits required 
%to encode a system state f. The encoding scheme transforms f from a number 
%in an arbitrary number system to numdigits double precision decimal 
%numbers.
%
% INPUT:  f is a row vector representing a finite-valued function which is 
%         a system state.
%
%         Tsize is a row vector representing a map from each state s in S 
%         to a component size, which is the number of states connected to
%         s (i.e. the cardinality of the subset Si containing s). The
%         coding scheme regards Tsize(f(i)) for each i as the base of the 
%         ith digit of f with f representing a number in an arbitrary 
%         number system.      
%
% OUTPUT: numdigits is the integer number of double precision digits
%         (decimal numbers) required to encode a function f.
%
%         startofdigit is a 1 by numdigits row vector which is an index 
%         into the first digit of f belonging to an encoded decimal digit 
%         such that startdigit(i) is the first digit of f belonging to the 
%         ith decimal digit.
%
%         endofdigit is a 1 by numdigits row vector which is an index into 
%         the last digit of f belonging to an encoded decimal digit such 
%         that startdigit(i) is the last digit of f belonging to the ith
%         decimal digit.
%
% See also, IDComponentTypes, Encode, Decode
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011

p = length(f);
digit = zeros(1,p);
digit(1) = 1;
maxval = Tsize(f(1));
for i = 2:p
    maxval = maxval*Tsize(f(i));
    if eps(maxval) > 1
        digit(i) = digit(i-1) + 1;
        maxval = Tsize(f(i));
    else
        digit(i) = digit(i-1);
    end
end
[~,startofdigit] = unique(digit,'first');
[~,endofdigit] = unique(digit,'last');
numdigits = max(digit);
end