function [F,T,W] = GenerateFullStateSpaceGraph(Q,rules,D,rho,finit,Tscript,Tsize,Trelab,numdigits,startofdigit,endofdigit)
%GENERATEFULLSTATESPACEGRAPH computes the full state-space reachability
%graph.
%
% INPUT:  Q is a cell array representing the set of component transitions
%         such that Q{j,1} represents the source states Xj, Q{j,2}
%         represents the mapping Yj from Xj to the destinations states in 
%         S, and Q{j,3} represents the multiplicity function Zj. 
%         Specifically, a transition j involves Q{j,3}(i) components in 
%         state Q{j,1}(i), each of which map to state Q{j,2}(i), for 
%         i = 1,...,length(Q{j,1}).
%
%         rules is a 4 column numeric array representing the transition 
%         rules. The first column contains indices into the set Q of 
%         component transitions, the second column contains indices into 
%         the set C of rate constants, the third column contains indices 
%         into the set D of dependency matrices, and the fourth column 
%         contains a flag indicating the format of its associated 
%         dependency matrix (zero indicates spatial coupling format, a 
%         nonzero indicates mean field coupling format). Each row j is a 
%         mapping from the component transition Q(rules(j,1)) to the rate 
%         constant C(rules(j,2)) under the restriction that the system
%         state satisfies dependency matrix D(rules(j,3)). If a rate 
%         constant cj in C is not associated with a dependency graph (i.e. 
%         has no functional dependencies), then rules(j,3) should be set to 
%         zero (rules(j,4) is ignored in this case).
%
%         D is a cell vector of dependency matrices. A dependency matrix in
%         spatial coupling format is given as a q by 3 cell array (where q 
%         is the number of hyperarcs in the dependency graph) with the 
%         first column containing the tail sets Ay, the second containing 
%         the head sets By, and the third containing the coloring function 
%         Ry of each yth hyperarc such that rho{Ry(j)} is the color of 
%         By(j). A dependency matrix in mean field coupling format is given
%         as a 2 row matrix with the first row containing indices into rho,
%         and the second containing an integer value such that D{i}(2,j) is
%         the number of components required to be in the state class 
%         D{i}(1,j).
%
%         rho is cell vector of component state classes. Each cell contains
%         a unique subset of the set S of component states.
%
%         finit is a row vector representing a finite-valued function which 
%         is the root state in the system reachability graph.
%
%         Tscript is a row vector representing the partition {S1,S2,...,Sr}
%         of the set S of component states such that the value of T(s) is 
%         the same for all s in Si and different for all s not in Si.
%
%         Tsize is a row vector representing a map from each state s in S 
%         to a component size, which is the number of states connected to
%         s (i.e. the cardinality of the subset Si containing s).
%
%         Trelab is a row vector representing a function which relabels 
%         each state s belonging to Si such that the set of labels are 
%         {0,1,...,Tsize(s)-1}.
%
%         numdigits is the integer number of double precision digits
%         (decimal numbers) required to encode a function f.
%
%         startofdigit is a 1 by numdigits row vector which is an index 
%         into the first digit of f belonging to an encoded decimal digit 
%         such that startdigit(i) is the first digit of f belonging to the 
%         ith decimal digit.
%
%         endofdigit is a 1 by numdigits row vector which is an index into 
%         the last digit of f belonging to an encoded decimal digit such 
%         that startdigit(i) is the last digit of f belonging to the ith
%         decimal digit.
%
% OUTPUT: F is an n by numdigits array of encoded system states
%         representing the full set of system states reachable from finit.
%
%         T is an m by 2 array of transitions between system states in F 
%         such that (T(i,1),T(i,1)) is the transition from state F(T(i,1)) 
%         to state F(T(i,2)).
%
%         W is an m by 1 array representing the weighting function of T
%         such that W(i) is the weight associated with transition T(i,:). 
%         Here, a weight W(i) is an index into a set of rate constants.
%
% Notes: This function calls the following functions:
%        Encode
%        Decode
%        cartprod
%        issubmultiset
%
% See also, GenerateReducedStateSpaceGraph
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References:
% 1. Bugenhagen, SM et al. Manuscript in preparation.

Fixtype = 'uint8';
Clen = length(unique(rules(:,2)));
if Clen <= 2^8-1
    Cixtype = 'uint8';
elseif Clen <= 2^16-1
    Cixtype = 'uint16';
elseif Clen <= 2^32-1
    Cixtype = 'uint32';
else
   error('rules is too large.');
end

blksize = 1e3;
unprocessed = zeros(blksize,1,Fixtype);
unprocessedixmax = blksize;
F = zeros(blksize,numdigits);
Fsort = zeros(blksize,numdigits);
Fptn = zeros(blksize,1,Fixtype);
Fptnends = zeros(blksize,1,Fixtype);
Fixmax = blksize;
Fsortixmax = blksize;

Tblksize = 1e4;
T = zeros(Tblksize,2,Fixtype);
W = zeros(Tblksize,1,Cixtype);
Tixmax = Tblksize;
Tixend = 0;

m = size(Q,1);
X = Q(:,1);
Y = Q(:,2);
Z = Q(:,3);

Tscriptunique = unique(Tscript);
numtypes = length(Tscriptunique);
Tscriptix = cell(1,numtypes);
for i = 1:numtypes
    Tscriptix{i} = find(Tscript(finit) == Tscriptunique(i));
end

F(1,:) = Encode(finit,Tsize,Trelab,numdigits,startofdigit,endofdigit);
Fixend = 1;
unprocessed(1) = 1;
unprocessedixend = 1;
Fsort(1,:) = Encode(sortf(finit,numtypes,Tscriptix),Tsize,Trelab, ...
                                        numdigits,startofdigit,endofdigit);
Fsortixend = 1;
Fptnends(1) = 1;
Fptn(1) = 1;

while any(unprocessed)
    f = Decode(F(unprocessed(unprocessedixend),:),finit,Tscript,Tsize, ...
                                 Trelab,numdigits,startofdigit,endofdigit);
    [funique,ends] = unique(sort(f),'last');
    mfunique = zeros(1,length(ends));
    mfunique(1) = ends(1);
    mfunique(2:end) = ends(2:end) - ends(1:end-1);
    fixF = unprocessed(unprocessedixend);
    unprocessed(unprocessedixend) = 0;
    unprocessedixend = unprocessedixend - 1;
    for j = 1:m
        if issubmultiset(X{j},Z{j},funique,mfunique)
            ixrules = find(rules(:,1)'==j);
            lenX = length(X{j});
            Dcombs = cell(1,lenX);
            for x = 1:lenX;
                Dcombs{x} = nchoosek(find(f == X{j}(x)),Z{j}(x));
            end
            Dcombs = cartprod(Dcombs);
            tf_sorted = false;
            tf_mfc = false;
            for i = 1:size(Dcombs,1)
                for k = ixrules
                    if ~rules(k,3)
                        tf = true;
                        break
                    elseif ~rules(k,4)
                        tf = checkdependencies_spc(f,Dcombs(i,:),...
                                                        D{rules(k,3)},rho);
                        if tf
                            break
                        end
                    elseif tf_mfc
                        tf = true;
                    else
                        tf = checkdependencies_mfc(f,D{rules(k,3)},rho);
                        if tf
                            ixrules = k;
                            tf_mfc = true;
                            break
                        end
                    end
                end
                if tf
                    fprime = f;
                    Tixend = Tixend + 1;
                    if Tixend > Tixmax
                        T = [T; zeros(Tblksize,2,Fixtype)];
                        W = [W; zeros(Tblksize,1,Cixtype)];
                        Tixmax = Tixmax + Tblksize;
                    end
                    W(Tixend) = rules(k,2);
                    for x = 1:lenX
                        ixDcombs = f(Dcombs(i,:)) == X{j}(x);
                        fprime(Dcombs(i,ixDcombs)) = Y{j}(x);
                    end
                    fprimecode = Encode(fprime,Tsize,Trelab,numdigits, ...
                                                  startofdigit,endofdigit);
                    if ~tf_sorted
                        fprimesort = sortf(fprime,numtypes,Tscriptix);
                        fprimesortcode = Encode(fprimesort,Tsize, ...
                                Trelab, numdigits,startofdigit,endofdigit);            
                        fprimeixFsort = find(all(bsxfun(@eq, ...
                              fprimesortcode, Fsort(1:Fsortixend,:)),2),1);
                        tf_sorted = true;
                    end
                    fprimeixF = 0;
                    if fprimeixFsort
                        if fprimeixFsort == 1
                            Fptnix = 1:Fptnends(1);
                        else
                            Fptnix = Fptnends(fprimeixFsort-1)+1: ...
                                                   Fptnends(fprimeixFsort);
                        end
                        fprimeixF = Fptn(Fptnix(find(all(bsxfun( ...
                                 @eq,fprimecode,F(Fptn(Fptnix),:)),2),1)));
                    end
                    if fprimeixF
                        T(Tixend,:) = [fixF,fprimeixF];
                    else
                        Fixend = Fixend + 1;
                        unprocessedixend = unprocessedixend + 1;
                        switch Fixend
                            case 2^8
                                Fixtype = 'uint16';
                                unprocessed = uint16(unprocessed);
                                Fptn = uint16(Fptn);
                                Fptnends = uint16(Fptnends);
                                fixF = uint16(fixF);
                                T = uint16(T);
                            case 2^16
                                Fixtype = 'uint32';
                                unprocessed = uint32(unprocessed);
                                Fptn = uint32(Fptn);
                                Fptnends = uint32(Fptnends);
                                fixF = uint32(fixF);
                                T = uint32(T);
                            case 2^32
                                error('F is too large');
                        end
                        if Fixend > Fixmax
                            F = [F; zeros(blksize,numdigits)];
                            Fptn = [Fptn; zeros(blksize,1,Fixtype)];
                            Fixmax = Fixmax + blksize;
                        end
                        if unprocessedixend > unprocessedixmax
                            unprocessed = [unprocessed; ...
                                                 zeros(blksize,1,Fixtype)];
                            unprocessedixmax = unprocessedixmax + blksize;
                        end
                        T(Tixend,:) = [fixF,Fixend];
                        F(Fixend,:) = fprimecode;
                        unprocessed(unprocessedixend) = Fixend;
                        if fprimeixFsort
                            Fptnends(fprimeixFsort:Fsortixend) = ...
                                    Fptnends(fprimeixFsort:Fsortixend) + 1;
                            Fptn(double(Fptnends(fprimeixFsort))+1: ...
                            Fixend) = Fptn(Fptnends(fprimeixFsort): ...
                                                                 Fixend-1);
                            Fptn(Fptnends(fprimeixFsort)) = Fixend;
                        else
                            Fsortixend = Fsortixend + 1;
                            fprimeixFsort = Fsortixend;
                            if Fsortixend > Fsortixmax
                                Fsort = [Fsort; zeros(blksize,numdigits)];
                                Fptnends = [Fptnends; ...
                                                 zeros(blksize,1,Fixtype)];
                                Fsortixmax = Fsortixmax + blksize;
                            end
                            Fsort(Fsortixend,:) = fprimesortcode;
                            Fptn(Fixend) = Fixend;
                            Fptnends(Fsortixend) = Fixend;
                        end   
                    end
                end
            end
        end
    end
end

F = F(1:Fixend,:);
T = T(1:Tixend,:);
W = W(1:Tixend);

end

function [tf] = checkdependencies_spc(f,A,D,rho)
    tf = false;
    for y = 1:size(D,1)
        if isequal(A,D{y,1})
            B = D{y,2};
            R = D{y,3};
            for x = 1:length(B)
                tf = ismembc(f(B(x)),rho{R(x)});
                if ~tf
                    break
                end
            end
            if tf
                break
            end
        end
    end
end

function [tf] = checkdependencies_mfc(f,D,rho)
    tf = false;
    for x = 1:size(D,2)
        tf = nnz(ismembc(sort(f),rho{D(1,x)})) == D(2,x);
        if ~tf
            break
        end
    end
end

function [f] = sortf(f,numtypes,Tscriptix)
    for i = 1:numtypes
        Tscriptixi = Tscriptix{i};
        f(Tscriptixi) = sort(f(Tscriptixi));
    end
end
