function [P,t] = SolveMasterEqtn_transient(F,T,W,M,C,odesolver,tspan,P0,options)
%SOLVEMASTEREQTN_TRANSIENT sets up and integrates the master equation 
%associated with a system's state-space graph using a built-in MATLAB ode
%solver.
%
% INPUT:  F is an array of system states (each row is a system state) 
%         representing a set, or a set of equivalence classes of, system 
%         states.
%         
%         T is an array of transitions between system states in F such that
%         (T(i,1),T(i,1)) is the transition from state F(T(i,1)) to state 
%         F(T(i,2)).
%
%         W is a size(T,1) vector representing the weighting function of T
%         such that W(i) is the weight associated with transition T(i,:). 
%         A weight W(i) is an index into the set C of rate constants.
%
%         M is a size(T,1) vector representing a multiplicity function of T
%         such that M(i) is the integer-valued multiplicity associated with 
%         transition T(i,:). In the case that the multiplicity function's 
%         range is singleton, M may be given as a scalar value.
%
%         C is a cell vector of numeric constants or function handles 
%         representing a set of specific probability rate constants.
%         A function handle is used in the case that a rate constant is a
%         (smooth) function of time.
%
%         odesolver is a function handle to one of MATLAB's built-in ode
%         solvers.
%
%         tspan is a vector of time points listing either: 1) the initial
%         and final time points; or 2) the specific time points at which
%         the solution is required.
%
%         P0 is a numeric vector such that sum(P0) = 1, representing the
%         initial probability distribution of the system states.
%
%         options is an options structure for changing parameters of the
%         ode solver.
%
% OUTPUT: P is the solution to the master equation such that P(i,:) is the
%         probability of the system occupying state F(i) at each time point 
%         of integration.
%
%         t is a vector of time points such that P(:,i) is the solution at
%         time t(i).
%
% See also, SolveMasterEqtn_SS
%        
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011

T = double(T);
M = double(M);
n = size(F,1);
m = size(T,1);
ixC = false(1,length(C));
Q = zeros(length(C),1);
for i = 1:length(C)
    if isa(C{i},'numeric')
        Q(i) = C{i};
    elseif isa(C{i},'function_handle')
        ixC(i) = true;
    else
        error('Cells in C must contain a number or a function handle.');
    end
end
ixC = find(ixC);
i = [T(:,2); (1:n)'];
j = [T(:,1); (1:n)'];
s = zeros(m+n,1);
if nargin < 9
    options = [];
end
[t,P] = odesolver(@odefun,tspan,P0,options);
t = t';
P = P';

%% DYDT function
function [dPdt] = odefun(t,P)
    for k = ixC
        Q(k) = C{k}(t);
    end
    s(1:m) = M.*Q(W);
    s(m+1:end) = 0;
    A = sparse(i,j,s,n,n);
    s(m+1:end) = -sum(A,1);
    A = sparse(i,j,s,n,n);
    dPdt = A*P;
end
end